@@ -1,12 +1,20 @@
 from __future__ import absolute_import
 
+import os
+
 import mock
 import pytest
 
+from keg.cli import dotenv, get_load_dotenv
+from keg.testing import CLIBase, app_config
 from keg_apps.cli import CLIApp
 from keg_apps.cli2.app import CLI2App
 from keg_apps.db.app import DBApp
-from keg.testing import CLIBase, app_config
+
+
+need_dotenv = pytest.mark.skipif(
+    not get_load_dotenv() or dotenv is None, reason='dotenv not supported'
+)
 
 
 class TestCLI(CLIBase):
@@ -29,6 +37,38 @@ class TestCLI(CLIBase):
         result = self.invoke(cmd_name='baz', exit_code=2)
         assert 'No such command \'baz\'' in result.output
 
+    @need_dotenv
+    def test_dotenv(self):
+        test_dir = os.path.dirname(__file__)
+
+        # Place dotenv file in search path of python-dotenv
+        flaskenv = os.path.abspath(os.path.join(test_dir, '..', '..', '.flaskenv'))
+        try:
+            with open(flaskenv, 'w') as f:
+                f.write('FOO=bar')
+            assert 'FOO' not in os.environ
+            self.invoke()
+            assert os.environ['FOO'] == 'bar'
+        finally:
+            os.remove(flaskenv)
+            del os.environ['FOO']
+
+    @need_dotenv
+    @mock.patch.dict(os.environ, {'FLASK_SKIP_DOTENV': '1'})
+    def test_disable_dotenv_from_env(self):
+        test_dir = os.path.dirname(__file__)
+
+        # Place dotenv file in search path of python-dotenv
+        flaskenv = os.path.abspath(os.path.join(test_dir, '..', '..', '.flaskenv'))
+        try:
+            with open(flaskenv, 'w') as f:
+                f.write('FOO=bar')
+            assert 'FOO' not in os.environ
+            self.invoke()
+            assert 'FOO' not in os.environ
+        finally:
+            os.remove(flaskenv)
+
     @pytest.mark.skipif(True, reason='reminder')
     def test_default_exception_handling(self):
         """
@@ -53,28 +93,45 @@ class TestCLI2(CLIBase):
     def test_no_commands_help_message(self):
         result = self.invoke()
         assert 'Usage: ' in result.output
-        assert '--quiet         Set default logging level to logging.WARNING' in result.output
+        assert '-q, --quiet     Increase logging level for reduced logging' in result.output
+        assert '-v, --verbose   Decrease logging level for increased logging' in result.output
         assert '--profile TEXT  Name of the configuration profile to use.' in result.output
         assert '--help-all      Show all commands with subcommands.' in result.output
-        assert 'develop       Developer info and utils.' in result.output
+        assert 'develop         Developer info and utils.' in result.output
         assert 'hello1' in result.output
 
-    def test_quiet(self):
+    @pytest.mark.parametrize('args,enabled', (
+        ([], {'warning', 'error', 'critical'}),
+        (['--quiet'], {'error', 'critical'}),
+        (['--quiet', '--quiet'], {'critical'}),
+        (['--quiet', '--quiet', '--quiet'], set()),
+        (['--quiet', '--quiet', '--quiet', '--quiet'], set()),
+        (['--verbose'], {'info', 'warning', 'error', 'critical'}),
+        (['--verbose', '--verbose'], {'debug', 'info', 'warning', 'error', 'critical'}),
+        (
+            ["--verbose", "--verbose", "--verbose"],
+            {"debug", "info", "warning", "error", "critical"},
+        ),
+        # Flags cancel each other out
+        (['--quiet', '--verbose'], {'warning', 'error', 'critical'}),
+    ))
+    def test_loglevel(self, args, enabled):
         # the asserts for these tests are in keg_apps.cli2.cli
-        result = self.invoke('--quiet', 'is-quiet')
+        levels = {'debug', 'info', 'warning', 'error', 'critical'}
+        disabled = levels - enabled
+        result = self.invoke(*args, 'test-log-level')
         assert 'printed foo' in result.output
-        assert 'logged foo' not in result.output
-
-        result = self.invoke('is-not-quiet')
-        assert 'printed foo' in result.output
-        assert 'logged foo' in result.output
+        for level in disabled:
+            assert f'{level} logged foo' not in result.output
+        for level in enabled:
+            assert f'{level} logged foo' in result.output
 
     def test_help_all(self):
         expected_lines = [
-            'Usage', '', 'Options', '--profile', '--quiet', '--help-all', '--help',
-            'Commands', 'develop', 'Commands', 'config', 'db', 'Commands', 'clear',
-            'init', 'routes',
-            'run', 'shell', 'templates', 'hello1', 'is-not-quiet', 'is-quiet', 'reverse',
+            'Usage', '', 'Options', '--profile', '-q, --quiet', '-v, --verbose',
+            '--help-all', '--help', 'Commands', 'develop', 'Commands', 'config',
+            'db', 'Commands', 'clear', 'init', 'routes',
+            'run', 'shell', 'templates', 'hello1', 'reverse', 'test-log-level',
             ''
         ]
         result = self.invoke('--help-all')
